#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
import argparse
import os
import subprocess
import sys

import compose_deploy


DOCKER_HOST_PATH = os.path.join(os.environ['HOME'], '.docker-remote.sock')

LISTEN_STRING = 'UNIX-LISTEN:{},reuseaddr,fork'.format(DOCKER_HOST_PATH)

SSH_COMMAND = 'ssh {} "socat STDIO UNIX-CONNECT:/var/run/docker.sock"'
REMOTE_STRING = 'EXEC:{}'.format(SSH_COMMAND)


def remote(server, command=None):
    os.environ['DOCKER_HOST'] = 'unix://{}'.format(DOCKER_HOST_PATH)
    os.environ['VIRTUALENVWRAPPER_PYTHON'] = '/usr/bin/python'

    if command is None:
        print('Starting a new shell with remote docker environment variable,\n'
              'exit this shell (usually ^D) as normal when you\'re done\n',
              file=sys.stderr)

    socket_forward_process = subprocess.Popen(
        ['/usr/bin/socat',
         LISTEN_STRING,
         REMOTE_STRING.format(server)
         ])

    to_run = [os.getenv('SHELL', '/bin/bash')]

    if command is not None:
        if isinstance(command, basestring):
            to_run.extend(['-c', command])
        else:
            to_run.extend(command)

    shell_process = subprocess.Popen(
        to_run,
        stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr)

    shell_process.wait()

    print("\nExited docker remote shell, cleaning up...", file=sys.stderr)

    socket_forward_process.terminate()
    socket_forward_process.wait()

    print("Done cleaning up", file=sys.stderr)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--version', '-V', action='store_true',
                        help='Print version number and exit')
    parser.add_argument('server',
                        help='Server to point docker at')
    args = parser.parse_args()

    if args.version:
        print(compose_deploy.__version__)
        return

    return remote(args.server)

if __name__ == '__main__':
    main()
